;+
; NAME: FASTPHOT_AREA
;
; PURPOSE: Perform the simultaneous PSF fitting of sources at known position.
;	   Use a space cut off and call FASTPHOT for sub extraction	    
;
; CALLING SEQUENCE: fastphot_area, map, psf, noise_map, X, Y, flux, $
;                        Fprior = Fprior, map_clean = map_clean
;
; INPUTS:
;         MAP: Array containing the map of the studied field (all
;              the good pixel must be >-99998). 
;              If bad pixel, put -99999. value
;         PSF: Array containing the PSF. It must be the (2N+1)*(2N+1)
;              image (in the unit of the map) for a source with flux = 1
;              (in output unit).
;         NOISE_MAP: Array containing the standard deviation of the
;                    noise. (same size as map) 
;	  SUBDIMX: Dimension on x direction of the subextracted map
;	  SUBDIMY: Dimension on y direction of the subextracted map
;	  EDGEX: Additionnal edge in x fitted with the submap (the flux of
;	         the sources on the edge are not saved).
;                Must be about 10 PSF FWHM.             
;	  EDGEY: same thing in y
;         X: Vector containing the x coordinates of the sources. (float if decimal coordinates or not)
;         Y: Vector containing the y coordinates of the sources. (float if decimal coordinates or not)
;
; KEYWORD PARAMETERS:
;         FPRIOR: Estimation of the flux of the fitted sources
;         (accelerate the fit)
;         MAP_CLEAN: Map cleaned from fitted sources.
;         (useful to check the results of a fit)
;         EXACT: Invert exactly the matrix (do not use conjugate
;         gradient method)
;
;         EFLUX: Vector containing the uncertainties on the output fluxes.
;
; OUTPUTS:
;         FLUX: flux of the fitted sources.
;
; MODIFICATION HISTORY:
; Written by Morgan Cousin in May 2009
; Documentation by Morgan Cousin in March 2010
;-

PRO fastphot_area, map, psf, noise_map, subdimx, subdimy, edgex, edgey, X, Y, flux, eflux = eflux, Fprior = Fprior, map_clean = map_clean, exact = exact	
  
  if keyword_set(alpha) ne 1 then alpha = 0.

;Compute the size of different elements
  dim1 = (size(map))(1)		;x size of the map
  dim2 = (size(map))(2)		;y size of the map
  dlx = subdimx + 2*edgex	;x size of subextracted map with revovery edge
  dly = subdimy + 2*edgey	;y size of subextracted map with revovery edge
  Lx = dim1
  Ly = dim2
  Nx = double(Lx)/double(subdimx)
  Ny = double(Ly)/double(subdimy)
;X size adjustment
  while (Nx - round(Nx)) ne 0. do begin
    Lx = Lx + 1
    Nx = double(Lx)/double(subdimx)
  endwhile
;Y size adjustment
  while (Ny - round(Ny)) ne 0. do begin
    Ly = Ly + 1
    Ny = double(Ly)/double(subdimy)
  endwhile
  Lx = Lx + 2*edgex			;Full x size of the map after adjustment
  Ly = Ly + 2*edgey			;Full y size of the map after adjustment
  Ns = n_elements(X)			;Number of sources in the map
  flux = fltarr(Ns)
  if keyword_set(eflux) then eflux = 1.+0.*flux

  if keyword_set(Fprior) ne 1 then Fprior = fltarr(Ns)		;If no prior flux in entrance

  fullmap = fltarr(Lx, Ly)
  fullmap(*,*) = -99999.		;Initialisation for detect bad pixels in the fallowing proress
  map_noise = fltarr(Lx, Ly) 
;inclusion de la map
  fullmap[edgex :edgex + dim1 - 1, edgey :edgey + dim2 - 1] = map
  map_noise[edgex :edgex + dim1 - 1, edgey :edgey + dim2 - 1] = noise_map
;Shift the sources positions to take into account the added edge
  Xs = X + edgex
  Ys = Y + edgey

  sub_map = fltarr(dlx, dly)
  sub_map_noise = fltarr(dly, dly)
  match = intarr(Ns)
  number_of_estimated_flux = 0

  for i = 0, Nx -1 do begin
    for j = 0, Ny -1 do begin
  
        ;----------------------------------------------------------------
	;Extraction of the sub_map
        sub_map(*,*) = 0.
	sub_map = fullmap[i*(dlx - 2*edgex) : i*(dlx - 2*edgex) + dlx -1, j*(dly - 2*edgey) : j*(dly - 2*edgey) + dly -1]
	;----------------------------------------------------------------
	;Extraction of the sub_map_noise
        sub_map_noise = map_noise[i*(dlx - 2*edgex) : i*(dlx - 2*edgex) + dlx -1, j*(dly - 2*edgey) : j*(dly - 2*edgey) + dly -1]
	 
	;Selection of sources present in the full sub_map (with edge)
	;----------------------------------------------------------------
        ind_sub_sources = where(Xs ge float(i*(dlx - 2*edgex)) and Xs le float(i*(dlx - 2*edgex) + dlx -1) $
				and Ys ge float(j*(dly - 2*edgey)) and Ys le float(j*(dly - 2*edgey) + dly -1))

        if ind_sub_sources(0) ne -1 then begin 

	    sub_Xs = Xs(ind_sub_sources) - float(i*subdimx) 	;Shift in x local coordinates
            sub_Ys = Ys(ind_sub_sources) - float(j*subdimy)	;Shift in y local coordinates
	    sub_Fprior = Fprior(ind_sub_sources)		;Prior flux selection

	    ;Selection of sources present in the internal sub_map
	    ;----------------------------------------------------------------
	    ind_sub_sources_flag = where(sub_Xs ge float(edgex) and sub_Xs lt float(edgex + subdimx) $
					and sub_Ys ge float(edgey) and sub_Ys lt float(edgey + subdimy))
  
	    if ind_sub_sources_flag(0) ne -1 then begin 
	      
	      sub_Ns = n_elements(ind_sub_sources_flag)  			;Number of local sources	      
	      number_of_estimated_flux = number_of_estimated_flux + sub_Ns	;Total number of estimated flux
	      match(ind_sub_sources(ind_sub_sources_flag)) = 1			;localised sources in the internal sub_map

              ;loadct_plot
              ;plot, xs, ys, psym = 3
              ;oplot, xs(ind_sub_sources), ys(ind_sub_sources), psym = 3, color = 1
	      ;oplot, xs(ind_sub_sources(ind_sub_sources_flag)), ys(ind_sub_sources(ind_sub_sources_flag)), psym = 1, color = 2

              ;stop

                                ;Extraction with prior on the sub_map
                                ;----------------------------------------------------------------
              sub_eflux = 1.
	      fastphot, sub_map, psf, sub_map_noise, sub_Xs, sub_Ys, sub_flux, eflux = sub_eflux, Fprior = sub_Fprior, exact = exact
              ;print, 'top', i, j
            
	      ;flux assignment
	      for k = 0, sub_Ns - 1 do begin
		  flux(ind_sub_sources(ind_sub_sources_flag(k))) = sub_flux(ind_sub_sources_flag(k)) 
                  if keyword_set(eflux) then eflux(ind_sub_sources(ind_sub_sources_flag(k))) = sub_eflux(ind_sub_sources_flag(k))
	      endfor
	    endif
        endif
    endfor ; end j
  endfor ;end i

;Completness test
  if number_of_estimated_flux ne Ns then begin
    print, 'Fail ::', 100.*float(number_of_estimated_flux)/float(Ns),'% of sources are analysed'
  endif else begin
    print, 'Done ::', 100.*float(number_of_estimated_flux)/float(Ns),'% of sources are analysed'
  endelse

;Clean map construction
;----------------------------------------------------------------
  if arg_present(map_clean) then begin
     map_clean = map		
     add_sources_interp, map_clean, X, Y, psf, flux*(-1.) 	
  endif

END
