;+
; NAME: FASTPHOT_GRADSOLVER
;
; PURPOSE: Compute the flux results of the AX = B equation for
; FASTPHOT with a conjugate gradient method.
;
; CALLING SEQUENCE: PRO fastphot_gradsolver, A, B, X_out, $
;                          X0=X0, Nloop = Nloop
;
; INPUTS: 
;         A: N*N matrix computed by FASTPHOT
;         B: N vector computed by FASTPHOT
;
; OPTIONAL INPUTS:
;                  X0: Initial condiction for X (can help to converge faster).
;                  Nloop: Number of interation for conjugate gradient algorithm.
;
; OUTPUTS:
;          X_out: Result of the equation.
;
; MODIFICATION HISTORY:
; Written by Morgan Cousin in May 2009
; Documented by Matthieu Bethermin in Jan 2010 
;-

PRO fastphot_gradsolver, A, B, X_out, X0=X0, $
                         err = err, Nloop = Nloop
  
  Ns = n_elements(B(*))
  
  if keyword_set(X0) ne 1 then begin 
     X0 = fltarr(Ns)
     X0(*)=0.
  endif
  
  X = dblarr(Ns,2)
  G = dblarr(Ns,2)
  H = dblarr(Ns,2)
  C = dblarr(NS,2)
  fin = dblarr(4)
  
  X(*,0) = double(X0(*))
  AX0 = A#X(*,0)
  G(*,0) = AX0(*)-B(*)          ;compute the first residual
  H(*,0) = -G(*,0)
  fin(0) = 1.e+10
  fin(1) = 1.e+10
  fin(2) = 1.e+10
  fin(3) = 0
  Nloop = 0
  while double(fin(0)) gt 1.e-9 xor fin(3) gt 60000 do begin   
     alphanum = total(G(*,0)*H(*,0))
     AH0 = A#H(*,0)
     alphaden = total(H(*,0)*AH0(*))
     if alphaden ne 0. then begin  
        alpha = -(alphanum/alphaden)
        X(*,1) = X(*,0)+alpha*H(*,0)
        G(*,1) = G(*,0)+alpha*AH0(*)
        betanum = total(G(*,1)*G(*,1))
        betaden = total(G(*,0)*G(*,0))
        beta = betanum/betaden
        H(*,1) = -G(*,1)+beta*H(*,0)
        X(*,0) = X(*,1)
        G(*,0) = G(*,1)
        H(*,0) = H(*,1)
     endif   
                                ;estimation of the convergence
     AX0 = A#X(*,0)
     C(*,1) = double(abs(AX0(*)-B(*)))
     fin(1) = double(median(C(*,1)))
     fin(2) = double(abs(fin(1)-fin(0)))
     fin(0) = fin(1)
     if double(fin(2)) lt 1.e-20 then begin
        fin(3) = 100000.
     endif
     C(*,0) = C(*,1) 
     Nloop = Nloop + 1
  endwhile
  if Nloop eq 60001 then begin
     print, 'Not converge'
     X(*,0) = 1.e10    
  endif
  X_out = X(*,0)
  if keyword_set(err) ne 1 then begin 
     err = fin(0)
  endif
  if keyword_set(Nloop) ne 1 then begin 
     Nloop = Nloop
  endif
END
